"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DVCClient = void 0;
var Variable_1 = require("./Variable");
var Request_1 = require("./Request");
var Store_1 = __importDefault(require("./Store"));
var User_1 = require("./User");
var EventQueue_1 = require("./EventQueue");
var utils_1 = require("./utils");
var EventEmitter_1 = require("./EventEmitter");
var RequestConsolidator_1 = require("./RequestConsolidator");
var DVCClient = /** @class */ (function () {
    function DVCClient(environmentKey, user, options) {
        var _this = this;
        this.store = new Store_1.default(window.localStorage);
        this.user = user;
        this.options = options;
        this.environmentKey = environmentKey;
        this.variableDefaultMap = {};
        this.eventQueue = new EventQueue_1.EventQueue(environmentKey, this, options === null || options === void 0 ? void 0 : options.flushEventsMS);
        this.requestConsolidator = new RequestConsolidator_1.RequestConsolidator();
        this.eventEmitter = new EventEmitter_1.EventEmitter();
        this.store.saveUser(this.user)
            .then(function () { return console.log('Successfully saved user to local storage!'); });
        this.onInitialized = (0, Request_1.getConfigJson)(environmentKey, this.user, options === null || options === void 0 ? void 0 : options.enableCloudData)
            .then(function (config) {
            var oldConfig = _this.config;
            _this.config = config;
            _this.store.saveConfig(config)
                .then(function () { return console.log('Successfully saved config to local storage'); });
            _this.eventEmitter.emitInitialized(true);
            _this.eventEmitter.emitFeatureUpdates((oldConfig === null || oldConfig === void 0 ? void 0 : oldConfig.features) || {}, _this.config.features);
            _this.eventEmitter.emitVariableUpdates((oldConfig === null || oldConfig === void 0 ? void 0 : oldConfig.variables) || {}, _this.config.variables, _this.variableDefaultMap);
            return _this;
        })
            .catch(function (err) {
            _this.eventEmitter.emitInitialized(false);
            _this.eventEmitter.emitError(err);
            return _this;
        });
    }
    DVCClient.prototype.onClientInitialized = function (onInitialized) {
        var _this = this;
        if (onInitialized && typeof onInitialized === 'function') {
            this.onInitialized
                .then(function () { return onInitialized(null, _this); })
                .catch(function (err) { return onInitialized(err); });
            return;
        }
        return this.onInitialized;
    };
    DVCClient.prototype.variable = function (key, defaultValue) {
        var _a;
        var _b, _c;
        var defaultValueKey = typeof defaultValue === 'string' ? defaultValue : JSON.stringify(defaultValue);
        if (this.variableDefaultMap[key] && this.variableDefaultMap[key][defaultValueKey]) {
            return this.variableDefaultMap[key][defaultValueKey];
        }
        var data = __assign({ key: key, defaultValue: defaultValue }, (_c = (_b = this.config) === null || _b === void 0 ? void 0 : _b.variables) === null || _c === void 0 ? void 0 : _c[key]);
        var variable = new Variable_1.DVCVariable(data);
        this.variableDefaultMap[key] = __assign((_a = {}, _a[defaultValueKey] = variable, _a), this.variableDefaultMap[key]);
        try {
            this.eventQueue.queueAggregateEvent({
                type: variable.isDefaulted
                    ? EventQueue_1.EventTypes.variableDefaulted
                    : EventQueue_1.EventTypes.variableEvaluated,
                target: variable.key,
            });
        }
        catch (e) {
            this.eventEmitter.emitError(e);
            console.log('Error with queueing aggregate events', e);
        }
        return variable;
    };
    DVCClient.prototype.identifyUser = function (user, callback) {
        var _this = this;
        var promise = new Promise(function (resolve, reject) {
            try {
                _this.eventQueue.flushEvents();
                var updatedUser_1;
                if (user.user_id === _this.user.user_id) {
                    updatedUser_1 = _this.user.updateUser(user);
                }
                else {
                    updatedUser_1 = new User_1.DVCPopulatedUser(user);
                }
                var oldConfig_1 = _this.config || {};
                _this.onInitialized.then(function () {
                    return _this.requestConsolidator.queue('identify', (0, Request_1.getConfigJson)(_this.environmentKey, updatedUser_1));
                }).then(function (config) {
                    _this.config = config;
                    _this.store.saveConfig(config)
                        .then(function () { return console.log('Successfully saved config to local storage'); });
                    var oldFeatures = oldConfig_1.features || {};
                    var oldVariables = oldConfig_1.variables || {};
                    _this.eventEmitter.emitFeatureUpdates(oldFeatures, config.features);
                    _this.eventEmitter.emitVariableUpdates(oldVariables, config.variables, _this.variableDefaultMap);
                    return config.variables;
                }).then(function (variables) {
                    _this.user = updatedUser_1;
                    _this.store.saveUser(updatedUser_1)
                        .then(function () { return console.log('Successfully saved user to local storage!'); });
                    return resolve(variables || {});
                }).catch(function (err) { return Promise.reject(err); });
            }
            catch (err) {
                _this.eventEmitter.emitError(err);
                reject(err);
            }
        });
        if (callback && typeof callback == 'function') {
            promise.then(function (variables) { return callback(null, variables); })
                .catch(function (err) { return callback(err, null); });
            return;
        }
        return promise;
    };
    DVCClient.prototype.resetUser = function (callback) {
        var _this = this;
        var anonUser = new User_1.DVCPopulatedUser({ isAnonymous: true });
        var promise = new Promise(function (resolve, reject) {
            try {
                _this.eventQueue.flushEvents();
                var oldConfig_2 = _this.config || {};
                _this.onInitialized.then(function () {
                    return _this.requestConsolidator.queue('identify', (0, Request_1.getConfigJson)(_this.environmentKey, anonUser));
                }).then(function (config) {
                    _this.config = config;
                    _this.user = anonUser;
                    _this.store.saveConfig(config).then(function () {
                        console.log('Successfully saved config to local storage');
                    });
                    _this.store.saveUser(anonUser).then(function () {
                        console.log('Successfully saved user to local storage!');
                    });
                    var oldFeatures = oldConfig_2.features || {};
                    var oldVariables = oldConfig_2.variables || {};
                    _this.eventEmitter.emitFeatureUpdates(oldFeatures, config.features);
                    _this.eventEmitter.emitVariableUpdates(oldVariables, config.variables, _this.variableDefaultMap);
                    resolve(config.variables || {});
                }).catch(function (e) {
                    reject(e);
                });
            }
            catch (e) {
                _this.eventEmitter.emitError(e);
                reject(e);
            }
        });
        if (callback && typeof callback == 'function') {
            promise.then(function (variables) { return callback(null, variables); })
                .catch(function (err) { return callback(err, null); });
            return;
        }
        return promise;
    };
    DVCClient.prototype.allFeatures = function () {
        var _a;
        return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.features) || {};
    };
    DVCClient.prototype.allVariables = function () {
        var _a;
        return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.variables) || {};
    };
    DVCClient.prototype.subscribe = function (key, handler) {
        this.eventEmitter.subscribe(key, handler);
    };
    DVCClient.prototype.unsubscribe = function (key, handler) {
        this.eventEmitter.unsubscribe(key, handler);
    };
    DVCClient.prototype.track = function (event) {
        var _this = this;
        (0, utils_1.checkParamDefined)('type', event.type);
        this.onInitialized.then(function () {
            _this.eventQueue.queueEvent(event);
        });
    };
    DVCClient.prototype.flushEvents = function (callback) {
        return this.eventQueue.flushEvents().then(function () { return callback === null || callback === void 0 ? void 0 : callback(); });
    };
    return DVCClient;
}());
exports.DVCClient = DVCClient;
//# sourceMappingURL=Client.js.map